import java.io.IOException;
import java.lang.*;
import java.io.*;
import java.lang.Integer;
import java.lang.Override;
import java.util.*;
import java.lang.String;
import java.util.Arrays;
import java.util.Comparator;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;

    interface Generator {
        void run() throws IOException;
    }

    static class TaskData {
        int t;
        int[] x;
        int[] ts;

        public TaskData(int t, int[] x, int[] ts) {
            this.t = t;
            this.x = x;
            this.ts = ts;
        }
    }

    Random random = new Random(0xDEADBEAF);
    int MAXX = 1_000_000_000;
    int MAXT = 1_000_000_000;
    
    void write(final TaskData data, String folder, String subtask, int points, String comment) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);
        out.println(data.x.length + " " + data.t);
        Integer[] p = new Integer[data.x.length];
        for (int i = 0; i < data.x.length; i++) {
            p[i] = i;
        }
        Arrays.sort(p, new Comparator<Integer>() {
            @Override
            public int compare(Integer a, Integer b) {
                int r = Integer.compare(data.x[a], data.x[b]);
                if (r == 0) {
                    return Integer.compare(data.x[a], data.x[b]);
                }
                return r;
            }
        });
        for (int i = 0; i < data.x.length; i++) {
            out.println(data.x[p[i]] + " " + data.ts[p[i]]);
        }
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=%s", path, folder, subtask, points, comment));
        System.out.println(String.format("Test %d is generated", testCount));
    }

    TaskData[] samples = {
            new TaskData(10, new int[]{1, 2, 3}, new int[]{4, 5, 3}),
            new TaskData(10, new int[]{1, 2, 3}, new int[]{2, 2, 3}),
            new TaskData(100, new int[]{8, 3, 1, 4, 5, 9, 100, 50}, new int[]{8, 10, 21, 3, 19, 32, 1, 1})
    };

    void genPreliminary() throws IOException {
        for (int i = 0; i < samples.length; i++) {
            write(samples[i], "preliminary", "subtask0", 0, String.format("\"sample %d\"", i + 1));
        }
        testCount = 0;
    }

    void genRandom(int n, int t, int minX, int maxX, int minT, int maxT) throws IOException {
        TaskData taskData = new TaskData(t, new int[n], new int[n]);

        for (int i = 0; i < n; i++) {
            taskData.x[i] = random.nextInt(maxX - minX + 1) + minX;
            taskData.ts[i] = random.nextInt(maxT - minT + 1) + minT;
        }

        write(taskData, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d\"", n));
    }

    void genOverflow(int n, int t) throws IOException {
        TaskData taskData = new TaskData(t, new int[n], new int[n]);
        System.arraycopy(overflowData.x, 0, taskData.x, 0, overflowData.x.length);
        System.arraycopy(overflowData.ts, 0, taskData.ts, 0, overflowData.ts.length);

        for (int i = overflowData.x.length; i < n; i++) {
            taskData.x[i] = random.nextInt(t / 2) + 1;
            taskData.ts[i] = random.nextInt(t / 2) + 1;
        }

        write(taskData, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"overflow, n=%d\"", n));
    }

    void genSubtask(Generator gen, int points, String comment) throws IOException {
        ++subtaskCount;

        gen.run();

        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
        testsDesc.println(comment);
    }

    TaskData overflowData = new TaskData(1_000_000_000,
            new int[]{1, 1_000_000_000},
            new int[]{999_999_999, 1_000_000_000});

    void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genPreliminary();

        // Subtask for O(2^n);
        genSubtask(new Generator() {
            @Override
            public void run() throws IOException {
                genRandom(5, MAXT, 1, MAXX, 1, MAXT);
                genRandom(10, MAXT, 1, MAXX, 1, MAXT);
                genRandom(10, MAXT, 1, MAXX, 1, MAXT);
                genRandom(20, MAXT, 1, MAXX, 1, MAXT);
                genRandom(20, MAXT, 1, 10, 1, MAXT);
                genRandom(20, MAXT, 1, 20000, 1, 20000);
                genRandom(20, MAXT, 1, 100000, 1, 100000);

                genRandom(20, 100, 1, 10, 1, 10);
                genRandom(20, 200, 1, 10, 1, 10);
                genRandom(20, 1000, 1, 10, 1, 10);
                genRandom(20, 1000, 1, 100, 1, 10);

                genRandom(20, 500000000, 1, MAXT, 1, MAXT);
                genRandom(20, 500000000, 1, MAXT, 1, MAXT);

                genRandom(20, MAXT, 1, 10000000, 1, 10000000);
                genRandom(20, MAXT, 1, 10000000, 1, 10000000);

                write(overflowData, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"overflow, n=2\""));
                genOverflow(20, 1_000_000_000);
            }
        }, 15, String.format("comment_subtask%d=\"n &lt; %d\"", subtaskCount + 1, 20));

        // Subtask for O(n * T)
        genSubtask(new Generator() {
            @Override
            public void run() throws IOException {
                genRandom(1000, 1000, 1, 1000, 1, 1000);
                genRandom(1000, 1000, 1, 1000, 1, 1000);
                genRandom(1000, 1000, 1, 20000, 1, 20000);
                genRandom(1000, 1000, 1, 100000, 1, 100000);

                genRandom(1000, 100, 1, 10, 1, 10);
                genRandom(1000, 200, 1, 10, 1, 10);
                genRandom(1000, 1000, 1, 10, 1, 10);
                genRandom(1000, 1000, 1, 100, 1, 10);

                genRandom(1000, 1000, 1, 10000000, 1, 10000000);
                genRandom(1000, 1000, 1, 10000000, 1, 10000000);
            }
        }, 20, String.format("comment_subtask%d=\"n &lt; %d, T &lt; %d\"", subtaskCount + 1, 1000, 1000));

        // Subtask for O(n^2);
        genSubtask(new Generator() {
            @Override
            public void run() throws IOException {
                genRandom(1000, MAXT, 1, MAXX, 1, MAXT);
                genRandom(1000, MAXT, 1, MAXX, 1, MAXT);
                genRandom(1000, MAXT, 1, 20000, 1, 20000);
                genRandom(1000, MAXT, 1, 100000, 1, 100000);

                genRandom(1000, 100, 1, 10, 1, 10);
                genRandom(1000, 200, 1, 10, 1, 10);
                genRandom(1000, 1000, 1, 10, 1, 10);
                genRandom(1000, 1000, 1, 100, 1, 10);

                genRandom(1000, 500000000, 1, MAXT, 1, MAXT);
                genRandom(1000, 500000000, 1, MAXT, 1, MAXT);

                genRandom(1000, MAXT, 1, 10000000, 1, 10000000);
                genRandom(1000, MAXT, 1, 10000000, 1, 10000000);

                write(overflowData, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"overflow, n=2\""));
                genOverflow(1000, 1000_000_000);
            }
        }, 25, String.format("comment_subtask%d=\"n &lt; %d\"", subtaskCount + 1, 1000));

        // Subtask for O(n log n);
        genSubtask(new Generator() {
            @Override
            public void run() throws IOException {
                genRandom(10000, MAXT, 1, MAXX, 1, MAXT);
                genRandom(20000, MAXT, 1, MAXX, 1, MAXT);
                genRandom(20000, MAXT, 1, MAXX, 1, MAXT);
                genRandom(100000, MAXT, 1, MAXX, 1, MAXT);
                genRandom(100000, MAXT, 1, MAXX, 1, MAXT);
                genRandom(100000, MAXT, 1, 20000, 1, 20000);
                genRandom(100000, MAXT, 1, 20000, 1, 20000);
                genRandom(100000, MAXT, 1, 20000, 1, 20000);
                genRandom(100000, MAXT, 1, 100000, 1, 100000);

                genRandom(100000, 100, 1, 10, 1, 10);
                genRandom(100000, 200, 1, 10, 1, 10);
                genRandom(100000, 1000, 1, 10, 1, 10);
                genRandom(100000, 1000, 1, 100, 1, 10);

                genRandom(100000, 500000000, 1, MAXT, 1, MAXT);
                genRandom(100000, 500000000, 1, MAXT, 1, MAXT);

                genRandom(100000, MAXT, 1, 10000000, 1, 10000000);
                genRandom(100000, MAXT, 1, 10000000, 1, 10000000);

                write(overflowData, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"overflow, n=2\""));
                genOverflow(100_000, 1000_000_000);
            }
        }, 40, String.format("comment_subtask%d=\"n &lt; %d\"", subtaskCount + 1, 100000));

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
