#include <cstdio>
#include <vector>
#include <algorithm>
#include <iostream>

using namespace std;

const int maxn = 1 << 17;

int n;
int a[maxn], p[maxn];

vector<int> fastsolve() {
    for (int i = 0; i < n; i++) p[i] = i;
    sort(p, p + n, [](int i, int j) { return a[i] > a[j]; });
    if (n % 2 == 1 && a[p[0]] == 1) {
        return vector<int>();
    }
    vector<int> ans(n, 2);
    int cur_sum = 0, cur_people = 0;
    for (int i = 0; i < n; i++) {
        if (cur_sum + a[p[i]] <= n - cur_people - 1) {
            cur_sum += a[p[i]];
            cur_people += 1;
            ans[p[i]] = 1;
        } else {
            for (int j = 0; j < n - cur_people - cur_sum; j++) {
                ans[p[n - j - 1]] = 1;
            }
            break;
        }
    }
    return ans;
}

bool reading() {
    if (scanf("%d", &n) != 1) return 0;
    for (int i = 0; i < n; i++) scanf("%d", &a[i]);
    return 1;
}

void print(vector<int> ans) {
    if (ans.empty()) printf("NO\n");
    else {
        printf("YES\n");
        for (int i = 0; i < (int)ans.size(); i++) printf("%d%c", ans[i], " \n"[i + 1 == n]);
    }
}

vector<int> slowsolve() {
    vector<vector<int> > p(n + 1, vector<int>(n + 1, -1));
    p[0][0] = 0;
    for (int i = 0; i < n; i++) {
        for (int j = 0; j <= n; j++) if (p[i][j] != -1) {
            p[i + 1][j] = 0;
            p[i + 1][j + a[i]] = 1;
        }
    }
    for (int i = 1; i <= n; i++) {
        if (p[i][n - i] != -1) {
            vector<int> res(n, 2);
            int ci = i, cj = n - i;
            while (ci > 0) {
                if (p[ci][cj] == 1) res[ci - 1] = 1;
                cj -= p[ci][cj] * a[ci - 1];
                ci--;
            }
            return res;
        }
    }
    return vector<int>();
}

bool correct(vector<int> b) {
    if (b.empty()) return 1;
    int cnt = 0, sum = 0;
    for (int i = 0; i < n; i++) {
        if (b[i] == 1) {
            cnt++;
            sum += a[i];
        }
    }
    return sum == n - cnt;
}

vector<int> slowestsolve() {
    for (int mask = 0; mask < 1 << n; mask++) {
        vector<int> b(n, 2);
        for (int i = 0; i < n; i++) if (mask & (1 << i)) b[i]--;
        if (correct(b)) return b;
    }
    return vector<int>();
}

bool check(vector<int> ans1, vector<int> ans2) {
    bool correct1 = correct(ans1);
    bool correct2 = correct(ans2);
    if (ans1.empty() != ans2.empty() || !correct1 || !correct2) return 0;
    return 1;
}

void gen() {
    while (1) {
        n = rand() % 5 + 1;
        for (int i = 0; i < n; i++) {
            a[i] = 0;
        }
        for (int i = 0; i < n; i++) {
            a[rand() % n]++;
        }
        if (*max_element(a, a + n) < n) break;
    }
}

void stress(bool f) {
    if (!f) return;
    for (int it = 0;; it++) {
        gen();
        vector<int> ans1 = fastsolve();
        vector<int> ans2 = slowestsolve();
        if (!check(ans1, ans2)) {
            print(ans1);
            cout << "instead of" << endl;
            print(ans2);
            cout << n << endl;
            for (int i = 0; i < n; i++) cout << a[i] << " \n"[i + 1 == n];
            slowsolve();
            exit(0);
        }
        cerr << it << " " << (ans1.empty() ? "NO" : "YES") << endl;
    }
}

int main() {
    freopen("parties.in", "r", stdin);
    freopen("parties.out", "w", stdout);

    stress(0);
    while (reading()) {
        print(fastsolve());
    }


    return 0;
}

